;*************************************************************************
;  	Source File:		TEMCON1.ASM	
;	Design & Code:		MPB 	
;	Date:			13-1-11
;	Version:		1.0
;	
;	Target Hardware:	TEMCON Board
;	Simualation:		Proteus VSM Ver 7.7
;	ISIS Design File:	TEMCON.DSN
;	
;*************************************************************************
;
;	Test program for PIC887 Controller Board
;
;	Circuit description:
;	PIC 16F887 flash microcontroller receives 4 analogue inputs from
;	temperature sensors (or test input pots) to control Heater, Vent 
;	and Fan in a target system such as a greenhouse. 
;	Target temp will be set up using keypad input and displayed 
;	on 2-digit multiplexed LED display.
;	
;	Test program:
;	Checks all inputs and outputs for correct hardware operation.
;	- Press keypad buttons 1-4 to display raw data from input pots
;	- Buttons 5-8 ditto
;	- Button 9 to sound buzzer
;	- Button * to operate HEATER output
;	- Button 0 to operate VENT output
;	- Button # to operate FAN output
;	
;	Configuration:
;   	-Internal clock mode (4MHz, 1us per instruction)
;	-Power-up timer enabled
;	-Watchdog timer disabled
;	-Code Protection off
;
; I/O ALLOCATION  ********************************************************
;
;	Analogue temp sensors input (10mV/degC)		RA0, RA1, RA2, RA5
;	ADC reference volts input (2.048V)		RA3
;	Buzzer output					RB0 = toggle
;	7-segment display 	Select lo digit		RB1 = 1
;				Select hi digit		RB2 = 1
;			  	Segments 		RC1 - RC7 = 1
;	Keypad column detect input (active low)		RD0 - RD2
;	Keypad row select output			RD3 - RD6 = 0
;	Relay interfaces	Heater 			RE0 = 1
;				Vent   			RE1 = 1
;	FET interface		Fan			RE2 = 1
;
;	Data direction codes:
;	TRISA = 11111111
;	TRISB = 11111000
;	TRISC = 00000001
;	TRISD = 00000111
;	TRISE = 00000000
;
;	RB3, RB6, RB7 reserved for ICD operation
;
; ADC SETUP **************************************************************
;
;	ADCON0	Bits 76  	01 = A/Dclock = f/8)
;		Bits 5432	Channel Select (AN0 - AN13)
;		Bit  2		Go=1 / Done=0
;		Bit  0		A/D module enable = 1
;	ADCON0 = 01xxx001 depending on channel required
;
;	ADCON1	Bit  7		1 = RIGHT justify result in ADRESH/ADRESL
;		Bits 3210	0010 = RA0-RA5 analogue, RE0-RE2 digital
;	ADCON1 = 00000010
;
; ASSEMBLER DIRECTIVES ***************************************************
;
	PROCESSOR 16F887	; Create list file and select processor
	INCLUDE "P16F887.INC"	; Include file containing register labels
;	0x00E4 = CONFIG CODE 1
;	0XFFFF = CONFIG CODE 2
;	..................................................................
;
count	EQU	020		; assign GPR1 for counter
;
;	..................................................................
;
; 	Set origin at address 000:
	org	0x000
;
; START PROGRAM **********************************************************

	nop			; No op. required at 000 for ICD mode

; Initialise control registers ...........................................
	
	banksel	ANSEL		; Select Bank 3
	movlw	b'00011111'	; Set Port A anlogue inputs
	movwf	ANSEL		; ..& Port E digital I/O
	clrf	ANSELH		; Set Port B digital I/O

	banksel	TRISB		; Select Bank 1
	movlw	b'11111000'	; Set up..
	movwf	PORTB		; display select & buzzer
	clrf	TRISC		; display outputs
	movlw	b'10000111'	; Set up..
	movwf	PORTD		; keypad I/O
	clrf	PORTE		; relay/FET outputs
	movlw	B'10000000'	; A/D right justified,
	movwf	ADCON1		; ..& internal ref
	movlw	0FF
	movwf	ADRESL

	banksel PORTC		; Select bank 0
	clrf	PORTC		; Clear display outputs
	movlw	B'01000001'	; A/D Fosc/8, select AN0,
	movwf	ADCON0		; ..& enable 


; Initialise outputs .....................................................

	banksel	PORTA		; select port data register bank 0
	clrf	PORTB		; switch off all outputs
	clrf	PORTD
	clrf	PORTE

	movlw	0FF		; 
	movwf	PORTC		; all display segments on
	goto	start		; jump over subroutines to main loop	

; Subroutine to wait about 0.8 ms ........................................

del8	clrf	count		; Load time delay of 256x3=768 us
again	decfsz	count		; Decrement and test counter
	goto	again		; until zero
	return			; 	 

; Subroutine to get analogue input .......................................

; Wait 20us ADC aquisition settling time ..

getAD	movlw	007		; Load time delay of 7x3=21 us
	movwf	count		; Load counter
down	decfsz	count		; Decrement and test counter
	goto	down		; until zero (3us per loop)	 

; Get analogue input ..

	bsf	ADCON0,GO	; Start A/D conversion
wait	btfsc	ADCON0,GO	; Wait for conversion to complete
	goto	wait		; by testing GO/DONE bit
	return			; from subroutine with result in ADRESL

; Subroutine to show ADC result ..........................................

show	nop
	banksel	ADRESL
	movf	ADRESL,W	; move ADC result
	banksel	PORTC
	movwf	PORTC		; to display
	return

; Subroutines to process keys ............................................

proc1	movlw	b'01000001'	; Select analogue channel 0
	movwf	ADCON0		; and
	call	getAD		; and get analogue input
	return			; for next key

proc2	movlw	b'01000101'	; Select analogue channel 1
	movwf	ADCON0		; and
	call	getAD		; and get analogue input
	return			; for next key

proc3	movlw	b'01001001'	; Select analogue channel 2
	movwf	ADCON0		; and
	call	getAD		; and get analogue input
	return			; for next key

proc4	movlw	b'01010001'	; Select analogue channel 4
	movwf	ADCON0		; and
	call	getAD		; and get analogue input
	return			; for next key

proc9	bsf	PORTB,0		; Toggle buzzer on
	call	del8		; delay about 0.8ms
	bcf	PORTB,0		; Toggle buzzer off
	call	del8		; delay about 0.8ms
	return			; for next key

procs	bsf PORTE,0		; switch on heater output
wait0	btfss PORTD,0		; * button released?
	goto wait0		; no, wait
	return			; 

proc0	bsf PORTE,1		; switch on vent output
wait1	btfss PORTD,1		; * button released?
	goto wait1		; no, wait
	return			; 

proch	bsf PORTE,2		; switch on fan output
wait2	btfss PORTD,2		; * button released?
	goto wait2		; no, wait
	return			; 

; Routine to scan keyboard ..........................................

scan	movlw	0FF		; Deselect...
	movwf	PORTD		; ...all rows on keypad

; scan row A of keypad ..........

	bcf	PORTD,3		; select row A of keypad

	btfsc	PORTD,0		; test key 1
	goto	key2		; next if not pressed
	call	proc1		; process key 1

key2	btfsc	PORTD,1		; test key 2
	goto	key3		; next if not pressed
	call	proc2		; process key 2

key3	btfsc	PORTD,2		; test key 2
	goto	key4		; next if not pressed
	call	proc3		; process key 3
	

; scan row B of keypad ..........

key4	bsf	PORTD,3		; deselect row A
	bcf	PORTD,4		; select row B

	btfsc	PORTD,0		; test key 4
	goto	key5		; next if not pressed
	call	proc4		; process key 4

key5	btfsc	PORTD,1		; test key 5
	goto	key6		; next if not pressed
	call	proc1		; process key 5

key6	btfsc	PORTD,2		; test key 6
	goto	key7		; next if not pressed
	call	proc2		; process key 6
	
; scan row C of keypad ..........

key7	bsf	PORTD,4		; deselect row B
	bcf	PORTD,5		; select row C

	btfsc	PORTD,0		; test key 4
	goto	key8		; next if not pressed
	call	proc3		; process key 4

key8	btfsc	PORTD,1		; test key 5
	goto	key9		; next if not pressed
	call	proc4		; process key 2

key9	btfsc	PORTD,2		; test key 2
	goto	keys		; next if not pressed
	call	proc9		; process key 3

; scan row D of keypad ..........

keys	bsf	PORTD,5		; deselect row C
	bcf	PORTD,6		; select row D

	btfsc	PORTD,0		; test key *
	goto	key0		; next if not pressed
	call	procs		; process key *

key0	btfsc	PORTD,1		; test key 0
	goto	keyh		; next if not pressed
	call	proc0		; process key 0

keyh	btfsc	PORTD,2		; test key #
	goto	done		; next if not pressed
	call	proch		; process key #
	
; all done ......................

done	bsf	PORTD,6		; deselect row D
	clrf	PORTE		; clear outputs
	return			; to main loop


; Main program ***********************************************************

start	bcf	PORTB,2		; switch off high digit of display
	bsf	PORTB,1		; and switch on low digit
	call 	scan		; and read keypad
	call	show		; and display

	bcf	PORTB,1		; switch off low digit of display
	bsf	PORTB,2		; and switch on high digit
	call 	scan		; and read keypad
	call	show		; and display

	goto 	start		; repeat main loop

	END			; of source code .........................


