; ******************************************************************
;  LOCK1.ASM                                       MPB   22-12-10
; ******************************************************************
;
; Four digit combination lock simulation demonstrates the hardware
; features of the DIZI demo board and the PIC 16F84A.
;
; Hardware:     DIZI Demo Board with PIC 16F84A (4MHz)
; Setup:        RA0-RA3         DIL Switch Inputs
;               RA4             Push Button Input / Analogue Input
;               RB0             Push Button Input / Audio Output
;               RB1-RB7         7-Segment Display Output
; Fuses:        WDT off, PuT on, CP off
;
; Operation --------------------------------------------------------
;
; To set the combination, a sequence of 4 digits is input on the DIL 
; piano switches; this is retained in the EEPROM when power is off. 
; To 'open' the lock, a sequence of 4 digits is input via
; the potentiometer. These are is compared with the stored data, and
; an audio output generated to indicate the correct sequence. 
; The processor halts if any digit fails to match, and the program 
; must be restarted.
;       To set a combination: 
;       1.      Hold Power On Button
;       2.      Press Button A
;       3.      Set a digit on DIL switches and Press A - beeps
;       4.      Repeat step 3 for 3 more digits  
;       5.      Release Power Button 
;
;       To check a combination: 
;       1.      Hold Power On Button
;       2.      Press Button B
;       3.      Set a digit on pot and Press B - beeps if matched
;       4.      Repeat step 3 for 3 more digits  
;                - if digits all match, siren is sounded
;                - if any digit fails to match, the processor halts
;       5.      Release Power Button 
;                
; *******************************************************************
        PROCESSOR 16F84A         ; Processor Type Directive
; *******************************************************************

; EQU:  Special Function Register Equates...........................

PCL     EQU     02      ; Program Counter Low
PORTA   EQU     05      ; Port A Data     
PORTB   EQU     06      ; Port B Data
STATUS  EQU     03      ; Flags
EEDATA  EQU     08      ; EEPROM Memory Data
EEADR   EQU     09      ; EEPROM Memory Address
EECON1  EQU     08      ; EEPROM Control Register 1
EECON2  EQU     09      ; EEPROM Control Register 2
 
; EQU: User Register Equates........................................

Period  EQU     0C      ; Period of Output Sound
Count   EQU     0D      ; Delay Down Counter
PotVal  EQU     0E      ; Analogue Input Value
DigVal  EQU     0F      ; Current Digit Value 00 to 09
 
; EQU: SFR Bit Equates..............................................

RP0     EQU     5       ; STATUS - Register Page Select
RD      EQU     0       ; EECON1 - EEPROM Memory Read Byte Initiate
WR      EQU     1       ; EECON1 - EEPROM Memory Write Byte Initiate
WREN    EQU     2       ; EECON1 - EEPROM Memory Write Enable
Z       EQU     2       ; STATUS - Zero Flag
 
; EQU: User Bit Equates.............................................
 
butA    EQU     4       ; PORTA  - RA4 Input Button
butB    EQU     0       ; PORTB  - RB0 Input Button
buzO    EQU     0       ; PORTB  - RB0 Output Buzzer

; *******************************************************************
 
; INIT: Initialise Port B (Port A defaults to inputs) 
 
   	MOVLW   001             ; RB0 = Input, RB1-RB7 = Outputs
	TRIS    PORTB           ; Set Data Direction
        MOVWF   PORTB           ; Clear Data
start  	GOTO    select          ; Select Combination Read or Write       

                    
 
; SUBROUTINES *******************************************************

; SUB1: 7-Segment Code Table using PCL + offset in W
;       Returns digit display codes, with '-' for numbers A to F

codtab  ADDWF   PCL             ; Add offset to Program Counter
        RETLW   B'01111110'     ; Return with display code for '0'
        RETLW   B'00001100'     ; Return with display code for '1'
        RETLW   B'10110110'     ; Return with display code for '2'
        RETLW   B'10011110'     ; Return with display code for '3'
        RETLW   B'11001100'     ; Return with display code for '4'
        RETLW   B'11011010'     ; Return with display code for '5'
        RETLW   B'11111010'     ; Return with display code for '6'
        RETLW   B'00001110'     ; Return with display code for '7'
        RETLW   B'11111110'     ; Return with display code for '8'
        RETLW   B'11011110'     ; Return with display code for '9'
        RETLW   B'10000000'     ; Return with display code for '-'
        RETLW   B'10000000'     ; Return with display code for '-'
        RETLW   B'10000000'     ; Return with display code for '-'
        RETLW   B'10000000'     ; Return with display code for '-'
        RETLW   B'10000000'     ; Return with display code for '-'
        RETLW   B'10000000'     ; Return with display code for '-'

; -------------------------------------------------------------------
; SUB2: Delay routine  
;       Receives delay count in W

delay   MOVWF   Count           ; Load counter from W
loop    DECFSZ  Count           ; and decrement
        GOTO    loop            ; until zero
        RETURN                  ; and return

; -------------------------------------------------------------------
; SUB3: Output One Beep Cycle to BuzO

beep    MOVLW   0FF             ; Load FF into     
        MOVWF   Period          ; Period counter

        MOVLW   B'00000000'     ; Set RB0 
        TRIS    PORTB           ; as output

; Do one cycle of rising tone....

cycle   BSF     PORTB,buzO      ; Output High
        MOVF    Period,W        ; Load W with Period value
        CALL    delay           ; and delay for Period
              
        BCF     PORTB,buzO      ; Output Low
        CALL    delay           ; and delay for same Period
        DECFSZ  Period          ; Decrement Period
        GOTO    cycle           ; and do next cycle until 0 
 
; Set RB0 to input again.........

        MOVLW   B'00000001'     ; Reset RB0 
        TRIS    PORTB           ; as input
        RETURN                  ; from tone cycle
        
; -------------------------------------------------------------------
; SUB4: Get pot value (Rv) using rise time due to C and R on RA4  
;       Returns with digit value (0-F) in DigVal
; Discharge external capacitor on RA4 
 
getpot  MOVLW   B'00001111'     ; Set RA4 
        TRIS    PORTA           ; as output  
        BCF     PORTA,4         ; and discharge C setting output low
        MOVLW   0FF             ; Delay for about 1ms
        CALL    delay           ; to ensure C is discharged 
        MOVLW   B'00011111'     ; Reset RA4
        TRIS    PORTA           ; as input

; Increment a counter until RA4 goes high due to charging of C
        
        CLRF    PotVal          ; Clear input value counter                  
check   INCF    PotVal          ; increment counter 
        MOVLW   03              ; Set delay count to 3
        CALL    delay           ; and delay between input checks
        BTFSS   PORTA,4         ; Check input bit RA4
        GOTO    check           ; and repeat if not yet high

; Mask out high bits of count value, and store & display 
; 4-bit digit value, 0-F   
         
        MOVF    PotVal,W        ; Put count value in W
        ANDLW   00F             ; and set high 4 bits to 0
        MOVWF   DigVal          ; Store 4-bit value
        CALL    codtab          ; Get 7-segment code, 0-9
        MOVWF   PORTB           ; and display

        RETURN                  ; with DigVal from setting of pot

; MAIN SEQUENCES ****************************************************

; SEQ1: Store 4 Digits in non volatile EEPROM 
;       Beep after each digit, and twice when 4 done

; Complete Button A input operation

stocom  MOVLW   0FF             ; Delay for about 1ms 
        CALL    delay           ; to avoid Button A switch bounce
        BTFSS   PORTA,butA      ; Wait for Button A 
        GOTO    stocom          ; to be released

; Read 4-bit binary number from DIL switches into EEDATA and display

        CLRF    EEADR           ; Zero EEPROM address register
getdil  MOVF    PORTA,W         ; Read DIL switches
        ANDLW   0F              ; and set high 4 bits to 0
        MOVWF   EEDATA          ; Put DIL value in EEPROM data
        
        CALL    codtab          ; Display DIL input as decimal
        MOVWF   PORTB           ;

        BTFSC   PORTA,butA      ; Check if Button A pressed
        GOTO    getdil          ; If not, keep reading DIL input 

; Store the current DIL input in EEPROM at current address

store   BSF     STATUS,RP0      ; Select Register Bank 1
        BSF     EECON1,WREN     ; Enable EEPROM write
        MOVLW   055             ; Write initialisation sequence 
        MOVWF   EECON2          ;
        MOVLW   0AA             ;
        MOVWF   EECON2          ;
        BSF     EECON1,WR       ; Write data into current address
        BCF     STATUS,RP0      ; Re-select Register Bank 0 
         
waita   BTFSS   PORTA,butA      ; Wait for Button A to be released       
        GOTO    waita           ; 
        CALL    beep            ; Beep to indicate digit write done
        
; Check if 4 digits have been stored yet, if not, get next         
        INCF    EEADR           ; Select next EEPROM address
        BTFSS   EEADR,2         ; Is the address now = 4?
        GOTO    getdil          ; If not, get next digit
        CALL    beep            ; Beep twice when 4 digits stored
        CALL    beep            ;
        GOTO    done            ; Go to sleep when done
; -------------------------------------------------------------------
; SEQ2: Check PotVal v EEPROM
 
checom  MOVLW   0FF             ; Delay for about 1ms 
        CALL    delay           ; to avoid Button B switch bounce
        BTFSS   PORTB,butB      ; Wait for Button B to be released   
        GOTO    checom          ;
        
; Read the value set on the input pot      
        
        CLRF    EEADR           ; Zero EEPROM address
potin   CALL    getpot          ; Get a digit value set on pot (Rv)
        BTFSC   PORTB,butB      ; Check in Button pressed again
        GOTO    potin           ; If not, keep reading the pot
        
; Get a digit value from EEPROM and compare with the pot input
        
        BSF     STATUS,RP0      ; Select Register Bank 1
        BSF     EECON1,RD       ; Read selected EEPROM location 
        BCF     STATUS,RP0      ; Re-select Register Bank 0
        MOVF    EEDATA,W        ; Copy EEPROM data to W
        XORWF   DigVal          ; Compare the input with EEPROM data
        BTFSS   STATUS,Z        ; If it does not match, go to sleep
        GOTO    done            ;

; If digit match obtained, check if 4 done and do next if not
 
waitb   BTFSS   PORTB,butB      ; Wait for Button B to be released
        GOTO    waitb           ;
        CALL    beep            ; Beep to confirm successful match

        INCF    EEADR           ; Select next EEPROM location
        BTFSS   EEADR,2         ; 4 digits checked yet?
        GOTO    potin           ; If not, do the next
        GOTO    siren           ; When 4 digits done, run siren

 
 
; *******************************************************************

; MAIN: Select Set or Check Combination

select  BTFSS   PORTA,butA      ; Button A pressed?
        GOTO    stocom          ; If so, store a combination
        BTFSS   PORTB,butB      ; Button B pressed?
        GOTO    checom          ; If so, check a combination
        GOTO    select          ; repeat endlessly
                                   
; *******************************************************************

; END1: When combination successfully matched, make siren sound 

siren   CALL    beep            ; Do a tone cycle
	GOTO	siren		; continuously

; -------------------------------------------------------------------

; END2: When a digit check fails, go to sleep, and try again

done    CLRF     PORTB          ; Switch off display 
        SLEEP                   ; Processor halts 

; *******************************************************************
        END                     ; of program source code

