; **********************************************************
;       INT1.ASM       MPB 	2-2-11     
; ***********************************************************
;
;       Minimal program to demonstrate interrupts.
;
;       An output binary count to LEDs on PortB, bits 1 - 7
;       is interrupted by an active low input at RB0/INT.
;       The Interrupt Service Routine sets all outputs high,
;       and waits for RA4 to go low before returning to
;       the main program.

;       Connect push button inputs to RB0 and RA4
;	ISIS simulation version, tested OK 
;       
;       Processor:      PIC 16F84A
;       Hardware:       Simulation version
;			(reset switch connected to RB0)
;       Clock:          CR ~100kHz
;       Inputs:         Push Buttons    
;                       RB0 = 0 = Interrupt
;                       RA4 = 0 = Return from Interrupt
;       Outputs:        RB1 - RB7: LEDs (active high)
;       
;       WDTimer:        Disabled
;       PUTimer:        Enabled
;       Interrupts:     RB0 interrupt enabled
;       Code Protect:   Disabled
;
;       Subroutines:    DELAY
;       Parameters:     None
;
; ***********************************************************

	PROCESSOR 16F84A	; Specify MCU type

;   Register Label Equates.................................. 
        
PORTA   EQU     05              ; Port A Data Register  
PORTB   EQU     06              ; Port B Data Register
INTCON  EQU     0B              ; Interrupt Control Register
timer   EQU     0C              ; GPR1 = delay counter
tempb   EQU     0D              ; GPR2 = Output temp. store 

;   Input Bit Label Equates ................................

intin   EQU     0               ; Interrupt input = RB0
resin   EQU     4               ; Restart input = RA4
INTF    EQU     1		; RB0 Interrupt Flag

; *************************************************************

;   Set program origin for Power On Reset................... 

        org     000             ; Program start address
        GOTO    setup           ; Jump to main program start

;   Interrupt Service Routine at address 004................ 

        org     004             ; ISR start address

        MOVF    PORTB,W         ; Save current output value
        MOVWF   tempb	      	; in temporary register
        
        MOVLW   b'11111111'     ; Switch LEDs 1-7 on
        MOVWF   PORTB		  

wait    BTFSC   PORTA,resin     ; Wait for restart input
        GOTO    wait	      	; to go low

        MOVF    tempb,w         ; Restore previous output
        MOVWF   PORTB	      	; at the LEDs
        BCF	INTCON,INTF     ; Clear RB0 interrupt flag
        
        RETFIE                  ; Return from interrupt 

;   DELAY subroutine........................................

delay   MOVLW   0xFF            ; Delay count literal is
        MOVWF   timer           ; loaded into spare register
down    DECFSZ  timer           ; Decrement timer register 
        GOTO    down            ; and repeat until zero then
        RETURN                  ; return to main program 


;   Main Program  ******************************************

;   Initialise Port B (Port A defaults to inputs)...........
        
setup   MOVLW   b'00000001'     ; Set data direction bits
        TRIS    PORTB     	; and load TRISB

        MOVLW   b'10010000'     ; Enable RB0 interrupt in
        MOVWF  	INTCON          ; Interrupt Control Register
        

;   Main output loop .......................................

count   INCF	PORTB	      	; Increment LED display
        CALL    delay           ; Execute delay subroutine
        GOTO    count           ; Repeat main loop always


        END                     ; Terminate source code

; **********************************************************


